/*
 * Copyright (C) 2002 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;

import javax.swing.ImageIcon;

/**
 * @author John David Ratliff
 * @version 1.0.0
 *
 * A simple ConnectFour game.
 */
public class ConnectFour {
    private DrawingWindow window;
    private int[][] board;
    private int turn = RED;
    private ImageIcon red, blue;
    private Point lastMove;
    private boolean winner = false;

    /**
     * The number of rows on the game grid.
     */
    public static final int ROWS = 10;
    
    /**
     * The number of columns on the game grid.
     */
    public static final int COLUMNS = 10;

    /**
     * The width of the game window.
     */
    public static final int WIDTH = 400;
    
    /**
     * The width of the game height.
     */
    public static final int HEIGHT = 400;

    /**
     * Constant for empty square on the game grid.
     */
    public static final int EMPTY = 0;
    
    /**
     * Constant for the red pieces on the game grid.
     */
    public static final int RED = 1;
    
    /**
     * Constant for the blue pieces on the game grid.
     */
    public static final int BLUE = 2;

    /**
     * The constructor for the game.
     */
    public ConnectFour() {
        // create the board array
        this.board = new int[ROWS][COLUMNS];

        // create the window
        this.window = new DrawingWindow("Connect Four", Color.black, WIDTH, HEIGHT);
        
        // setup the winner font
        this.window.getPen().setFont(new Font(null, Font.PLAIN, 48));

        // create the checker icons
        this.red = new ImageIcon("images/redchecker.gif");
        this.blue = new ImageIcon("images/bluechecker.gif");

        // initialize the board
        this.initializeStatus();

        // draw the board
        this.redrawBoard();

        // play the game
        this.takeTurnClick();
    }

    /**
     * Initializes the game board.
     */
    private void initializeStatus() {
        for (int i = 0; i < this.board.length; i++) {
            for (int j = 0; j < this.board[0].length; j++) {
                this.board[i][j] = EMPTY;
            }
        }
    }

    /**
     * Gets the proper checker image for the color.
     * 
     * @param color The color of the checker (RED or BLUE)
     * @return The Image Icon of the checker.
     */
    private ImageIcon getCheckerImage(int color) {
        if (color == RED) {
            return red;
        }

        return blue;
    }

    /**
     * Draws the checker board.
     */
    private void redrawBoard() {
        Graphics2D pen = this.window.getPen();
        int rowSpace = (WIDTH / ROWS);
        int columnSpace = (HEIGHT / COLUMNS);
        
        // erase the screen.
        pen.setPaint(Color.black);
        pen.fillRect(0, 0, WIDTH, HEIGHT);

        // draw the lines in white
        pen.setPaint(Color.white);

        // paint the row lines on the grid
        for (int i = 1; i < ROWS; i++) {
            pen.drawLine(0, (i * rowSpace), WIDTH, (i * rowSpace));
        }

        // paint the column lines on the grid
        for (int i = 1; i < COLUMNS; i++) {
            pen.drawLine((i * columnSpace), 0, (i * columnSpace), HEIGHT);
        }

        // paint the board pieces
        for (int i = 0; i < ROWS; i++) {
            for (int j = 0; j < COLUMNS; j++) {
                if (this.board[i][j] != EMPTY) {
                    ImageIcon checker = getCheckerImage(this.board[i][j]);
                    pen.drawImage(checker.getImage(), (i * rowSpace), (j * columnSpace), null);
                }
            }
        }

        // repaint the window...
        this.window.repaint();
    }

    /**
     * Check for a winner in a given row.
     * 
     * @param row The row to check.
     * @return EMPTY if there is no winner, or the color of winning side
     *          RED or BLUE
     */
    private int checkWinnerRow(int row) {
        for (int i = 0; i < (COLUMNS - 3); i++) {
            if ((this.board[row][i] != EMPTY) &&
                (this.board[row][i] == this.board[row][i+1]) &&
                (this.board[row][i] == this.board[row][i+2]) &&
                (this.board[row][i] == this.board[row][i+3])) {
                return this.board[row][i];
            }
        }

        // return EMPTY if no winner
        return EMPTY;
    }

    /**
     * Check for a winner in a given row.
     * 
     * @param col The column to check.
     * @return EMPTY if there is no winner, or the color of the
     *          winning side (RED or BLUE).
     */
    private int checkWinnerCol(int col) {
        for (int i = 0; i < (ROWS - 3); i++) {
            if ((this.board[i][col] != EMPTY) &&
                (this.board[i][col] == this.board[i+1][col]) &&
                (this.board[i][col] == this.board[i+2][col]) &&
                (this.board[i][col] == this.board[i+3][col])) {
                return this.board[i][col];
            }
        }

        // return EMPTY if no winner
        return EMPTY;
    }

    /**
     * Check for a winner in the diagonal rows.
     * 
     * @param row The x coordinate to start checking from.
     * @param col The y coordinate to start checking from.
     * @return EMPTY if no winner, or the color of the winning
     *          side (RED or BLUE).
     */
    private int checkWinnerDiag(int row, int col) {
        int x = row, y = col;
        
        // move to top-left
        while ((x > 0) && (y > 0)) {
            x--;
            y--;
        }
        
        // check down-right arrows
        while ((x < (ROWS - 3)) && (y < (COLUMNS - 3))) {
            if ((this.board[x][y] != EMPTY) &&
                (this.board[x][y] == this.board[x+1][y+1]) &&
                (this.board[x+1][y+1] == this.board[x+2][y+2]) &&
                (this.board[x+2][y+2] == this.board[x+3][y+3])) {
                return this.board[row][col];
            }
            
            x++;
            y++;
        }
        
        // reset for next test
        x = row;
        y = col;
        
        // move to top-right
        while ((x < (ROWS - 1)) && (y > 0)) {
            x++;
            y--;
        }
        
        // check down-left arrows
        while ((x > 0) && (y < (COLUMNS - 3))) {
            if ((this.board[x][y] != EMPTY) &&
                (this.board[x][y] == this.board[x-1][y+1]) &&
                (this.board[x-1][y+1] == this.board[x-2][y+2]) &&
                (this.board[x-2][y+2] == this.board[x-3][y+3])) {
                return this.board[row][col];
            }
            
            x--;
            y++;
        }
        
        // return EMPTY if there is no winner
        return EMPTY;
    }

    /**
     * Paint the winner message on the game grid.
     * 
     * @param color The color of the winning side.
     */
    private void paintWinner(int color) {
        Graphics2D pen = this.window.getPen();

        if (color != EMPTY) {
            pen.setPaint(Color.white);
            pen.fillRect(WIDTH / 4 - 15, HEIGHT / 2 - 48, 250, 60);

            if (color == RED) {
                pen.setPaint(Color.red);
                pen.drawString("Red Wins", WIDTH / 4, HEIGHT / 2);
            } else {
                pen.setPaint(Color.blue);
                pen.drawString("Blue Wins", WIDTH / 4, HEIGHT / 2);
            }

            this.window.repaint();
            winner = true;
        }
    }

    /**
     * Checks the board for a winner.
     */
    private void checkWinner() {
        paintWinner(checkWinnerRow(lastMove.x));
        paintWinner(checkWinnerCol(lastMove.y));
        paintWinner(checkWinnerDiag(lastMove.x, lastMove.y));
    }

    /**
     * Take a game turn by placing a piece in the column.
     * 
     * @param column The column to play the piece.
     */
    public void takeTurn(int column) {
        Graphics2D pen = this.window.getPen();
        ImageIcon checker;
        int rowSpace = (WIDTH / ROWS);
        int columnSpace = (HEIGHT / COLUMNS);

	if ((column < 0) || (column >= COLUMNS)) {
		System.out.println("Invalid column!");
		return;
	}

        for (int i = this.board[column].length - 1; i >= 0; i--) {
            if (this.board[column][i] == EMPTY) {
                // we can put our piece here...
                this.board[column][i] = turn;

                // switch the turn
                if (turn == RED) {
                    turn = BLUE;
                } else {
                    turn = RED;
                }

                // redraw the entire board
                this.redrawBoard();

                // reset the last move
                this.lastMove = new Point(column,i);

                // stop the loop
                i = -1;
            }
        }
    }

    /**
     * Plays the game taking turns using mouse clicks.
     */
    public void takeTurnClick() {
        Point p;
        int rowSpace = (WIDTH / ROWS);

        while ((p = this.window.getMouseClick()) != null && !winner) {
            this.takeTurn(p.x / rowSpace);
            this.checkWinner();
        }
    }

    /**
     * Start a new Connect Four game.
     * 
     * @param args Not used.
     */
    public static void main(String[] args) {
        new ConnectFour();
    }
}
