/** 
 * Creates a framed canvas in a window with a particular title,
 * background color, and size.
 *
 * Copyright (C) 2002 Suzanne Menzel
 * All Rights Reserved
 * 
 * @author Suzanne Menzel
 */

import java.awt.*;
import java.awt.event.*;

public class DrawingWindow extends Frame {
  
  private Image buf = null;
  private Insets insets;
  private Graphics2D pen;
  private Color backColor;
  private boolean first, polling;
  private int width, height;
  private MouseHandler mouse;
  
  private static int windowNum = 1;
  private static final int DEFAULT_WIDTH = 500;
  private static final int DEFAULT_HEIGHT = 500;
  private static final Color DEFAULT_COLOR = Color.white;
  
  public DrawingWindow() {
    this("Drawing Window #" + windowNum, DEFAULT_COLOR, 
         DEFAULT_WIDTH, DEFAULT_HEIGHT);
  }
  
  public DrawingWindow(Color backColor) {
    this("Drawing Window #" + windowNum, backColor, 
         DEFAULT_WIDTH, DEFAULT_HEIGHT);
  }
  
  public DrawingWindow(Color backColor, int width, int height) {
    this("Drawing Window #" + windowNum, backColor, width, height);
  }
  
  public DrawingWindow(String title) {
    this(title, DEFAULT_COLOR, DEFAULT_WIDTH, DEFAULT_HEIGHT);
  }
  
  public DrawingWindow(String title, int width, int height) {
    this(title, DEFAULT_COLOR, width, height);
  }
  
  public DrawingWindow(String title, Color backColor) {
    this(title, backColor, DEFAULT_WIDTH, DEFAULT_HEIGHT);
  }
  
  public DrawingWindow(int width, int height) {
    this("Drawing Window #" + windowNum, DEFAULT_COLOR, width, height);
  }
  
  public DrawingWindow(String title, Color backColor, 
                       int width, int height) {
    super(title);
    first = true;
    setBackground(backColor);
    this.backColor = backColor;
    this.width = width;
    this.height = height;
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        DrawingWindow.this.dispose();
        first = true;
        polling = false;
      }
    });
    ++windowNum;
    mouse = new MouseHandler();
    addMouseListener(mouse);
    addMouseMotionListener(mouse);
  }
  
  public int width() {
    return width;
  }
  
  public int height() {
    return height;
  }
    
  private void init() {
    first = false;
    show();
    insets = getInsets();
                       
    super.setSize(width + insets.left + insets.right,
                  height + insets.top + insets.bottom);
    show();   
                       
    setResizable(false);
    buf = createImage(width, height);
    pen = (Graphics2D) buf.getGraphics();
    pen.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                         RenderingHints.VALUE_ANTIALIAS_ON);
    clear();
    pen.setColor(Color.black);
  }
  
  public void clear() {
    Color save = pen.getColor();
    pen.setColor(backColor);
    pen.fillRect(0, 0, width, height);
    pen.setColor(save);
  }
  
  public void update(Graphics g) {
    paint(g);
  }
  
  public void paint(Graphics g) {
    toFront();
    if (buf != null)
      getGraphics().drawImage(buf, insets.left, insets.top, this);
  }
  
  public void repaint() {
    toFront();
    if (first) 
      init();
    getGraphics().drawImage(buf, insets.left, insets.top, this);
  }
  
  public Graphics2D getPen() {
    if (first) 
      init();
    return pen;
  }
  
  public void setSize(Dimension d) {
    super.setSize(d);
    init();
  }
  
  public Point getMouseClick() {
    toFront();
    requestFocus();
    Point p;
    // wait for a click by polling the mouse until a click occurs
    polling = true;  // set to false if the window is closed
    while ((p = mouse.getClick()) == null && polling)
      ;
    if (!polling)
      return null;
    p = new Point(p.x - insets.left, p.y - insets.top);
    return p;
  }
  
  public Point getMousePosition() {
    toFront();
    requestFocus();
    Point p = mouse.getPosition();
    if (p == null)
      p = new Point(0, 0);
    p = new Point(p.x - insets.left, p.y - insets.top);
    return p;
  }
  
  // some inner classes:
  class MouseHandler extends MouseAdapter 
    implements MouseMotionListener {
    private Point pos, click;
    public void mouseMoved(MouseEvent e) {
      pos = e.getPoint();
    }
    public void mousePressed(MouseEvent e) {
      click = e.getPoint();
    }
    public Point getPosition() {
      return pos;
    }
    public Point getClick() {
      Point temp = click;
      click = null;
      return temp;
    }
    public void mouseDragged(MouseEvent e) {}
  }
  
  public String toString() {
    return "DrawingWindow[title=" + getTitle() +
      ",backColor=" + backColor + 
      ",width=" + width +
      ",height=" + height + "]";
  }
}
